#!/bin/sh
#
# argument 1: option, reload webserver config, update ntp, ...

source /usr/local/bin/functions.sh

trap "on_exit $proc_arg0 $proc_arg1 $reporting_lock; exit 99" INT QUIT TERM

function on_exit() {
    # proc_arg0
    if [ "$1" != "undefined" -a -f "$1" ]; then
        rm -f $1
    fi
    # proc_arg1
    if [ "$2" != "undefined" -a -f "$2" ]; then
        rm -f $2
    fi
    # reporting_lock
    if [ "$3" != "undefined" -a -f "$3" ]; then
        rm -f $3
    fi
}

function check_and_lock_process() {
    proc_arg="$1"

    if [ -f "$proc_arg" ]; then
        # the lock file already exists, so what to do?
        if [ "$(ps -p `cat $1 2>/dev/null` 2>/dev/null | wc -l)" -lt 2 ]; then
            # process not running, but lock file not deleted?
            rm -f $proc_arg 2>/dev/null
            echo $$ > $proc_arg
            if [ -f "$proc_arg" ]; then
                return 0
            else
                return 1
            fi
        else
            return 2
        fi
    else
        echo $$ > $proc_arg
        if [ -f "$proc_arg" ]; then
            return 0
        else
            return 1
        fi
    fi
}

if [ "$1" == "" ]; then
    exit 1
fi

proc_arg0="undefined"
proc_arg1="undefined"
reporting_lock="undefined"

case $1 in
    soffice)
        su -c "setsid nice -1 /usr/lib/libreoffice/program/soffice.bin --headless --accept=\"socket,host=127.0.0.1,port=8100;urp;\" --nofirststartwizard --nocrashreport --nodefault --norestore --nologo --invisible 2>/dev/null <&2 >&2 &" www-data
    ;;
    ssl)
        if [ "$2" != "" -a "$3" != "" ]; then
            if [ $2 == "write" ]; then
                check_and_lock_process "/tmp/config.sh.ap.ssl"
                if [ $? -ne 0 ]; then
                    log_error "Couldn't lock ap.ssl process. Exiting..."
                    exit 10
                fi
                proc_arg0="/tmp/config.sh.ap.ssl"
            fi 
            case $2 in
                check)
                    md5sum $3 | cut -d" " -f1 | tr -d '\n\r'
                ;;
                read)
                    cat $3
                ;;
                check_pp)
                    ret0=`openssl rsa -noout -modulus -in $3 | openssl md5`
                    ret1=`openssl x509 -noout -modulus -in $4 | openssl md5`
                    if [ "$ret0" != "$ret1" ]; then
                        log_error "Public/Private do not match. Exiting..."
                        exit 10
                    fi
                ;;
                del_csr)
                    if [ "$3" != "" ]; then
                        if [ -n "$(echo $3 | grep '\.csr')" ]; then
                            rm /home/root/ssl_cert/requests/$3
                            del_file="/home/root/ssl_cert/requests/"$3
                            echo $del_file | sed "s/\.csr/\.key/" | xargs rm
                            echo $del_file | sed "s/\.csr/\.der/" | xargs rm
                        elif [ -n "$(echo $3 | grep '\.crt')" ]; then
                            rm $3
                            echo $3 | sed "s/\.crt/\.key/" | xargs rm
                            echo $3 | sed "s/\.crt/\.der/" | xargs rm
                            echo $3 | sed "s/\.crt/\.pem/" | xargs rm
                        fi
                    fi
                ;;
                cert)
                    if [ "$4" != "" -a "$5" != "" -a "$6" != "" -a "$7" != "" ]; then
                        dir_req="/home/root/ssl_cert/ss_certs"
                        if [ ! -d $dir_req ]; then
                            mkdir $dir_req
                            if [ $? -ne 0 ]; then
                                log_error "Couldn't create self signed directory. Exiting..."
                                exit 10
                            fi
                        fi
                        ret=`openssl req -x509 -nodes $7 -days $3 -newkey rsa:$4 -keyout $dir_req/$6.key -out $dir_req/$6.crt -config $5 -extensions v3_ca`
                        if [ $? -ne 0 ]; then
                            log_error "Couldn't create self-signed certificate "
                            exit 32
                        fi
                        ret=`openssl x509 -in $dir_req/$6.crt -text -noout`
                        if [ $? -ne 0 ]; then
                            rm -f $dir_req/$6.key
                            rm -f $dir_req/$6.crt
                            log_error "Couldn't verify self-signed certificate"
                            exit 33
                        fi
                        ret=`openssl rsa -in $dir_req/$6.key -check`
                        if [ $? -ne 0 ]; then
                            rm -f $dir_req/$6.key
                            rm -f $dir_req/$6.crt
                            log_error "Couldn't check self-signed certificate key"
                            exit 33
                        fi
                        ret=`openssl x509 -inform pem -outform der -in $dir_req/$6.crt -out $dir_req/$6.der`
                        if [ $? -ne 0 ]; then
                            log_error "Couldn't convert to DER format"
                            exit 33
                        fi
                        cat $dir_req/$6.crt > $dir_req/$6.pem
                        cat $dir_req/$6.key >> $dir_req/$6.pem
                    fi
                ;;
                csr)
                    if [ "$4" != "" -a "$5" != "" -a "$6" != "" ]; then
                        dir_req="/home/root/ssl_cert/requests"
                        if [ ! -d $dir_req ]; then
                            mkdir $dir_req
                            if [ $? -ne 0 ]; then
                                log_error "Couldn't create request directory. Exiting..."
                                exit 10
                            fi
                        fi
                        ret=`openssl req -new $5 -config $3 -newkey rsa:$6 -keyout $dir_req/$4.key -out $dir_req/$4.csr`
                        if [ $? -ne 0 ]; then
                            log_error "Couldn't create ssl signing request"
                            exit 32
                        fi
                        ret=`openssl req -inform pem -outform der -in $dir_req/$4.csr -out $dir_req/$4.der`
                        if [ $? -ne 0 ]; then
                            log_error "Couldn't convert to DER format"
                            exit 32
                        fi
                        ret=`openssl req -text -noout -verify -in $dir_req/$4.csr`
                        if [ $? -ne 0 ]; then
                            rm -f $dir_req/$4.key
                            rm -f $dir_req/$4.csr
                            log_error "Couldn't verify ssl signing request"
                            exit 33
                        fi
                    fi
                ;;
                write)
                    # $3 == server.pem, $4 backup cert, $5 new cert $6 serverCrt $7 ServerKey
                    if [ -f "$3" ]; then
                        mv $3 $4
                    fi
                    mv $5 $3
                    chmod 444 $3
                    chmod +x /home/root/ssl_cert/
                    chown root:root $3
                    # reporting installed ?
                    if [ "$6" != "" -a "$7" != "" ]; then
                        rm -f /var/lib/postgresql/9.1/main/server.crt /var/lib/postgresql/9.1/main/server.key
                        rm -f $6 $7
                        touch $6 2>/dev/null
                        touch $7 2>/dev/null

                        found_begining=0
                        found_end_crt=0
                        found_end_key=0
                        while read LINE
                        do
                            if [ $found_begining -eq 0 ]; then
                                if echo $LINE | grep "BEGIN " > /dev/null
                                then
                                    found_begining=1
                                    if echo $LINE | grep "CERTIFICATE" > /dev/null
                                    then
                                        ind_crt=1
                                    else
                                        ind_crt=2
                                    fi
                                else
                                    continue
                                fi
                            fi
                            if [ $ind_crt -eq 1 ]; then
                                if [ $found_end_crt -eq 0 ]; then
                                    echo $LINE >> $6
                                fi
                                if echo $LINE | grep "END CERTIFICATE" > /dev/null
                                then
                                    found_end_crt=1
                                    found_begining=0
                                    ind_crt=0
                                fi
                            fi
                            if [ $ind_crt -eq 2 ]; then
                                if [ $found_end_key -eq 0 ]; then
                                    echo $LINE >> $7
                                fi
                                if echo $LINE | egrep -e "(END RSA PRIVATE KEY|END PRIVATE KEY)" > /dev/null
                                then
                                    found_end_key=1
                                    found_begining=0
                                    ind_crt=0
                                fi
                            fi
                        done < $3
                        if [ $found_end_crt -eq 0 ]; then
                            log_msg "Error: SSL certificate not parsed correctly"
                            echo "-----END CERTIFICATE-----" >> $6
                        fi
                        if [ $found_end_key -eq 0 ]; then
                            log_msg "Error: SSL private key not parsed correctly"
                            echo "-----END RSA PRIVATE KEY-----" >> $7
                        fi

                        ln -s $6 server.crt
                        ln -s $7 server.key
                        mv server.crt /var/lib/postgresql/9.1/main
                        mv server.key /var/lib/postgresql/9.1/main
                        chown postgres:postgres $6 $7
                        chmod 400 $6 $7
                        if [ ! -L "/var/lib/postgresql/9.1/main/server.crt" -o ! -L "/var/lib/postgresql/9.1/main/server.key" ]; then
                            log_msg "Warning: SSL certificate installed but it may not work for reporting"
                        fi
                    fi
                ;;
            esac
        fi
    ;;
    cron)
        if [ $2 != "" ]; then
            if [ $2 == "write" ]; then
                check_and_lock_process "/tmp/config.sh.ap.cron"
                if [ $? -ne 0 ]; then
                    log_error "Couldn't lock ap.cron process. Exiting..."
                    exit 11
                fi
                proc_arg0="/tmp/config.sh.ap.cron"
            fi
            case $2 in
                restart)
                    /etc/init.d/cron restart 1>/dev/null
                ;;
                read)
                    if [ $3 != "" ]; then
                        cat $3
                    fi
                ;;
                write)
                    if [ $3 != "" -a $4 != "" ]; then
                        mv $4 $3
                        chown root:root $3
                        chmod 644 $3
                    fi
                ;;
            esac
        fi
    ;;
    shutdown)
        shift
        if [ "$2" != "" ]; then
            /sbin/shutdown $@
        else
            /sbin/shutdown -a -t 5 0 2>&1
        fi
    ;;
    ip-config)
        shift
        dhcp=0
        ip=""
        mask=""
        gateway=""
        dns1=""
        dns2=""
        while [ $# -gt 0 ]
        do
            case "$1" in
                -d) dhcp=$2; shift;;
                -a) ip=$2; shift;;
                -g) gateway=$2; shift;;
                -m) mask=$2; shift;;
                -d1) dns1=$2; shift;;
                -d2) dns2=$2; shift;;
            esac
            shift
        done
        if [ $dhcp -eq 0 ]; then
            if [ "$ip" == "" -o "$mask" == "" ]; then
                log_error "with static IP, -a (IP address) and -m (Network Mask) parameters must be set!"
                exit 52
            fi
        fi
        iffile="/etc/network/interfaces"
        echo "# Used by ifup(8) and ifdown(8). See the interfaces(5) manpage or
# /usr/share/doc/ifupdown/examples for more information.

#Loopback device
auto lo
iface lo inet loopback

#eth0 device
auto eth0" > $iffile
        if [ $dhcp -eq 1 ]; then    # DHCP client
            echo "iface eth0 inet dhcp" >> $iffile
        else                        # static IP
            echo "iface eth0 inet static" >> $iffile
            echo "        address $ip" >> $iffile
            echo "        netmask $mask" >> $iffile
            if [ "$gateway" != "" ]; then
                echo "        gateway $gateway" >> $iffile
            fi
            dnsfile="/etc/resolv.conf"
            echo -n "" > $dnsfile
            if [ "$dns1" != "" ]; then
                echo "nameserver $dns1" >> $dnsfile
            fi
            if [ "$dns2" != "" ]; then
                echo "nameserver $dns2" >> $dnsfile
            fi
        fi
        # restart eth0 interface
        sleep 2s
        ifdown eth0
        ifup eth0
    ;;
    reload)
    # argument 2: new config 3: webserver config
        check_and_lock_process "/tmp/config.sh.ap.reload"
        if [ $? -ne 0 ]; then
            log_error "Couldn't lock ap.reload process. Exiting..."
            exit 12
        fi
        proc_arg0="/tmp/config.sh.ap.reload"
        if [ "$2" != "" -a "$3" != "" ]; then
            mv -f $2 $3
            # sleep that web page can reload
            sleep 2s
            chown root:root $3
        fi
        /etc/init.d/lighttpd stop
        # make sure lighttpd is stopped
        while pidof lighttpd &>/dev/null
        do
            sleep 1;
        done
        # make sure php5 is stopped
        count=0
        while pidof php5-cgi &>/dev/null && test $count -lt 5
        do
            sleep 1;
            count=$((count+1))
        done
        if [ -n "$(pidof php5-cgi)" ]; then
            ps auxwww | grep -v "grep" | egrep -e "php5-cgi" | awk '{print $2}' | xargs kill -9
        fi
        # make sure fcgi is stopped
        count=0
        while pidof process.fcgi &>/dev/null && test $count -lt 5
        do
            sleep 1;
            count=$((count+1))
        done
        if pidof process.fcgi 1>/dev/null
        then
            ps auxwww | grep -v "grep" | egrep -e "process.fcgi" | awk '{print $2}' | xargs kill -9
        fi
        if [ ! -s "/var/run/lighttpd.pid" ]; then
            rm -f /var/run/lighttpd.pid
        fi
        /etc/init.d/lighttpd start
    ;;
    timezone)
        if [ "$2" != "" ]; then
            check_and_lock_process "/tmp/config.sh.ap.timezone"
            if [ $? -ne 0 ]; then
                log_error "Couldn't lock ap.timezone process. Exiting..."
                exit 13
            fi
            proc_arg0="/tmp/config.sh.ap.timezone"
            case $2 in
                check)
                    if ! ls -l /usr/share/zoneinfo/$3 1>/dev/null
                    then
                        log_error "timezone $3 does not exist. Exiting..."
                        exit 14
                    fi
                ;;
                change)
                    rm /etc/localtime
                    ln -s /usr/share/zoneinfo/$3 /etc/localtime
                    echo $3 > /etc/timezone
                    /etc/init.d/cron restart 1>/dev/null 
                ;;
            esac 
        fi
    ;;
    ntp)
    # argument 2: ntp server
        if [ "$2" != "" ]; then
            ntp_conf_dhcp_file="/var/lib/ntp/ntp.conf.dhcp"
            ntp_conf_file="/etc/ntp.conf"
            if [ $2 == "add" -o $2 == "delete" ]; then
                check_and_lock_process "/tmp/config.sh.ap.ntp"
                if [ $? -ne 0 ]; then
                    log_error "Couldn't lock ap.ntp process. Exiting..."
                    exit 15
                fi
                proc_arg0="/tmp/config.sh.ap.ntp"
            fi
            case $2 in
                read)
                    if ! ps axwww | grep ntpd | grep -v "grep" >/dev/null
                    then
                        log_error "ntpd process not running"
                        exit 16
                    else
                        cat $ntp_conf_file | egrep -e "^server [0-9.]+.*$" | cut -d" " -f2 | tr -d " "
                        if [ -f $ntp_conf_dhcp_file ]; then
                            cat $ntp_conf_dhcp_file | egrep -e "^server [0-9.]+.*$" | cut -d" " -f2 | tr -d " "
                        fi
                    fi
                    ;;
                add)
                    if [ $3 != "" ]; then
                        t1=`egrep -e "^server $3" < $ntp_conf_file`
                        t2=""
                        if [ -f $ntp_conf_dhcp_file ]; then
                            t2=`egrep -e "^server $3" < $ntp_conf_dhcp_file`
                        fi
                        if [ ! -n "$t1" -a ! -n "$t2" ]; then
                            echo "server $3 iburst" >> $ntp_conf_file
                            if [ -f $ntp_conf_dhcp_file ]; then
                                echo "server $3 iburst" >> $ntp_conf_dhcp_file
                            fi
                            /etc/init.d/ntp restart >/dev/null
                        else
                            echo "NTP server already in the list or server received through DHCP"
                        fi
                    fi
                    ;;
                delete)
                    if [ $3 != "" ]; then
                        tmp_conf=`mktemp /tmp/ntp.XXXXX`
                        if [ $? -ne 0 ]; then
                            log_error "Can't create temp file. Exiting..."
                            exit 17
                        fi
                        if egrep -e "^server $3" < $ntp_conf_file >/dev/null
                        then
                            sed -e "/$3/ d" < $ntp_conf_file > $tmp_conf
                            cp $tmp_conf $ntp_conf_file
                            chmod 600 $ntp_conf_file
                        fi
                        if [ -f $ntp_conf_dhcp_file ]; then
                            if egrep -e "^server $3" < $ntp_conf_dhcp_file >/dev/null
                            then
                                sed -e "/$3/ d" < $ntp_conf_dhcp_file > $tmp_conf
                                cp $tmp_conf $ntp_conf_dhcp_file
                                chmod 600 $ntp_conf_dhcp_file
                            fi
                        fi
                        rm $tmp_conf
                    fi
                    ;;
            esac
        fi
    ;;
    rpcap)
    # argument 3: stop, start, check if running
        if [ "$2" != "" ]; then
            check_and_lock_process "/tmp/config.sh.ap.rpcap"
            if [ $? -ne 0 ]; then
                log_error "Couldn't lock ap.rpcap process. Exiting..."
                exit 18
            fi
            proc_arg0="/tmp/config.sh.ap.rpcap"
            case $2 in
                stop) ps auxwww | grep -v grep | grep rpcapd | awk '{ print $2 }' | xargs kill -9
                ;;
                start) /usr/local/bin/rpcapd -p 2002 -4 -n -d > /dev/null 2>&1
                ;;
                restart)
                    ps auxwww | grep -v grep | grep rpcapd | awk '{ print $2 }' | xargs kill -9
                    /usr/local/bin/rpcapd -p 2002 -4 -n -d > /dev/null 2>&1
                ;;
                check) ps auxwww | grep -v grep | grep rpcapd | wc -l
                ;;
            esac
        fi
    ;;
    rsyslog)
        /etc/init.d/rsyslog rotate > /dev/null
        if [ $? -ne 0 ]; then
            log_error "rsyslog couldn't be reloaded. Exiting..."
            exit 60
        fi
    ;;
    debian-packages)
        if [ "$2" != "" -a "$3" != "" ]; then
            if [ "$2" == "view" ]; then
                # dpkg --get-selections >> $3
                dpkg -l | egrep -e "^([ \t]*ii).*" | cut -c 5- > $3
                echo "lighttpd-mod-webdav                  1.4.35                                        WebDAV module for lighttpd" >> $3
                echo "lighttpd-mod-websocket               1.4.35                                        Websocket module for lighttpd" >> $3
                echo "lighttpd                             1.4.35                                        A fast webserver with minimal memory footprint" >> $3
            else
                dpkg -l | egrep -e "^([ \t]*ii).*" | cut -c 5- > $3
                echo "lighttpd-mod-webdav                  1.4.35                                        WebDAV module for lighttpd" >> $3
                echo "lighttpd-mod-websocket               1.4.35                                        Websocket module for lighttpd" >> $3
                echo "lighttpd                             1.4.35                                        A fast webserver with minimal memory footprint" >> $3
                unix2dos -f $3
            fi
        fi
    ;;
    postgres)
    # argument 3: backup, restore
        if [ "$2" != "" ]; then
            check_and_lock_process "/tmp/config.sh.ap.postgres"
            if [ $? -ne 0 ]; then
                log_error "Couldn't lock postgres process. Exiting..."
                exit 19
            fi
            proc_arg0="/tmp/config.sh.ap.postgres"
            case $2 in
                vacuum-full)
                    if [ "$3" != "" ]; then
                        /usr/bin/sudo -u postgres nice -1 vacuumdb -f -d "$3"
                        /usr/bin/sudo -u postgres nice -1 reindexdb -d "$3"
                    fi
                ;;
                restart)
                    /etc/init.d/postgresql restart
                ;;
                delete-ip)
                    if [ "$3" != "" -a "$4" != "" -a "$5" ]; then
                        tmp_conf=`mktemp -q /tmp/$(basename $0).XXXXXX`
                        if [ $? -ne 0 ]; then
                            log_error "Can't create temp file. Exiting..."
                            on_exit $proc_arg0 $proc_arg1
                            exit 20
                        fi
                        sed -e 's;[ \t]*host[ \t]*'"$3"'[ \t]*'"$4"'[ \t]*'"$5"'/32[ \t]*md5.*$;;' < /etc/postgresql/9.1/main/pg_hba.conf > tmp_conf
                        cp tmp_conf /etc/postgresql/9.1/main/pg_hba.conf
                        /etc/init.d/postgresql restart &>/dev/null
                        rm -f tmp_conf
                    else
                        log_error "database, username or ip is missing"
                        on_exit $proc_arg0 $proc_arg1
                        exit 21
                    fi
                ;;     
                add-ip)
                    if [ "$3" != "" -a "$4" != "" -a "$5" ]; then
                        if ! grep "host  $3  $4  $5/32  md5" < /etc/postgresql/9.1/main/pg_hba.conf &>/dev/null
                        then
                            echo "host  $3  $4  $5/32  md5" >> /etc/postgresql/9.1/main/pg_hba.conf
                            /etc/init.d/postgresql restart &>/dev/null
                        fi
                    else
                        log_error "database, username or ip is missing"
                        on_exit $proc_arg0 $proc_arg1
                        exit 22
                    fi
                ;;
                check-pwd)
                    if [ "$3" != "" -a "$4" != "" ]; then
                        chk_pwd=`sudo -u postgres psql -c "SELECT * FROM pg_shadow WHERE passwd='md5' || MD5('$3' || '$4')" postgres`
                        if ! echo $chk_pwd | grep "$4" &>/dev/null
                        then
                            log_error "$chk_pwd"
                            on_exit $proc_arg0 $proc_arg1
                            exit 23
                        fi
                    fi
                ;;
                password)
                    if [ "$3" != "" -a "$4" != "" ]; then
                        resPass=`sudo -u postgres psql -c "ALTER user \"$4\" WITH password '$3'" $4 2>&1`
                        if ! echo $resPass | grep "ALTER ROLE" &>/dev/null
                        then 
                            log_error "$resPass"
                            on_exit $proc_arg0 $proc_arg1
                            exit 24
                        fi
                    else
                        log_error "no new password provided"
                        on_exit $proc_arg0 $proc_arg1
                        exit 25
                    fi
                ;; 
                db-backup)
                   if [ "$3" != "" ]; then
                       /usr/bin/sudo -u postgres /usr/bin/pg_dump --encoding=utf8 --schema=public -Fc -U postgres $3 | gzip -fc6
                   else
                       log_error "please enter a valid database name"
                       on_exit $proc_arg0 $proc_arg1
                       exit 26
                   fi 
                ;;
                db-restore)
                    if [ "$3" != "" -a "$4" != "" -a "$5" != "" ]; then
                        update_seq_file=""
                        if [ $5 == "innovaphone" ]; then
                            update_seq_file="/home/postgres/restore/update_innovaphone_seq.psql"
                        elif [ $5 == "innovaphone-exchange" ]; then
                            update_seq_file="/home/postgres/restore/update_exchange_seq.psql"
                        elif [ $5 == "innovaphone-faxserver" ]; then
                            update_seq_file="/home/postgres/restore/update_faxserver_seq.psql"
                        elif [ $5 == "innovaphone-reporting" ]; then
                            update_seq_file="/home/postgres/restore/update_reporting_seq.psql"
                            reporting_lock="/tmp/config.sh.reporting.postgres"
                            echo $$ > $reporting_lock
                            chmod 777 $reporting_lock
                        fi
                        if [ ! -f $update_seq_file ]; then
                            log_error "$update_seq_file does not exist"
                            on_exit $proc_arg0 $proc_arg1 $reporting_lock
                            exit 27
                        fi
                        tconfig=`mktemp -q /tmp/$(basename $0).XXXXXX`
                        if [ $? -ne 0 ]; then
                            log_error "Can't create temp file. Exiting..."
                            on_exit $proc_arg0 $proc_arg1 $reporting_lock
                            exit 28
                        fi
                        tbackup=`mktemp -q /tmp/$(basename $0).XXXXXX`
                        if [ $? -ne 0 ]; then
                            rm -f $tconfig
                            log_error "Can't create temp file. Exiting..."
                            on_exit $proc_arg0 $proc_arg1 $reporting_lock
                            exit 28
                        fi
                        # dump actual db
                        /usr/bin/sudo -u postgres /usr/bin/pg_dump -s --encoding=utf8 --schema=public -Fc -f $3 $5 2>$tconfig
                        /usr/bin/sudo -u postgres psql -c "REVOKE CONNECT ON DATABASE \"$5\" FROM PUBLIC,\"$5\";" 1>/dev/null 2>>$tconfig
                        /usr/bin/sudo -u postgres /usr/bin/psql -c "SELECT pg_terminate_backend(pg_stat_activity.procpid) FROM pg_stat_activity WHERE pg_stat_activity.datname = '$5';" 1>/dev/null 2>>$tconfig
                        /usr/bin/sudo -u postgres dropdb $5 2>>$tconfig
                        if [ -s $tconfig ]; then
                            cat $tconfig
                            log_error "Can't delete existing database. Exiting..."
                            rm -f $tconfig
                            rm -f $tbackup
                            on_exit $proc_arg0 $proc_arg1 $reporting_lock
                            exit 29
                        fi
                        tmpdb="$5.tmp"
                        /usr/bin/sudo -u postgres /usr/bin/createdb --template=template0 --encoding=utf8 -U postgres $tmpdb 2>$tconfig && \
                        /usr/bin/sudo -u postgres psql -c "ALTER DATABASE \"$tmpdb\" OWNER TO \"$5\"" 1>/dev/null 2>$tconfig && \
                        /usr/bin/sudo -u postgres psql -c "GRANT ALL PRIVILEGES ON DATABASE \"$tmpdb\" to \"$5\"" 1>/dev/null 2>$tconfig && \
                        /usr/bin/sudo -u $5 /usr/bin/pg_restore --schema=public -s --dbname=$tmpdb $3 -U $5 2>$tconfig
                        # create files with psql commands for dropping and creating all contraints/indexes
                        add_constraints=`mktemp /tmp/psqladd.XXXXX`
                        add_indexes=`mktemp /tmp/psqladd.XXXXX`
                        add_indexes2=`mktemp /tmp/psqladd.XXXXX`
                        new_indexes=`mktemp /tmp/psqladd.XXXXX`
                        drop_states=`mktemp /tmp/psqldrop.XXXXX`
                        chmod 755 /tmp/psql*
                        db_error=0
                        psql_result=`/usr/bin/sudo -u postgres psql -1 -t -q -f /home/postgres/restore/get_add_constraint_statement.psql --dbname=$tmpdb -v ON_ERROR_STOP= > $add_constraints`
                        if [ $? -ne 0 ]; then
                            log_error "reading constraints failed: $psql_result"
                            db_error=1
                        fi
                        psql_result=`/usr/bin/sudo -u postgres psql -1 -t -q -f /home/postgres/restore/get_add_index_statement.psql --dbname=$tmpdb -v ON_ERROR_STOP= > $add_indexes`
                        if [ $? -ne 0 ]; then
                            log_error "reading indexes failed: $psql_result"
                            db_error=1
                        fi
                        psql_result=`/usr/bin/sudo -u postgres psql -1 -t -q -f /home/postgres/restore/get_drop_statements.psql --dbname=$tmpdb -v ON_ERROR_STOP= > $drop_states`
                        if [ $? -ne 0 ]; then
                            log_error "reading drop statements failed: $psql_result"
                            db_error=1
                        fi
                        # now drop all constraints and indexes
                        psql_result=`/usr/bin/sudo -u postgres psql -1 -t -q -f $drop_states --dbname=$tmpdb -v ON_ERROR_STOP= 2>&1`
                        if [ $? -ne 0 ]; then
                            log_error "dropping constraints and indexes failed: $psql_result"
                            db_error=1
                        fi
                        if [ $5 == "innovaphone-reporting" ]; then
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t cdrs --dbname=$tmpdb $4 2>$tconfig && \
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t events --dbname=$tmpdb $4 2>$tconfig && \
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t cdr_properties --dbname=$tmpdb $4 2>$tconfig && \
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t groups --dbname=$tmpdb $4 2>$tconfig && \
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t filters --dbname=$tmpdb $4 2>$tconfig && \
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t filter_times --dbname=$tmpdb $4 2>$tconfig && \
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t filter_conditions --dbname=$tmpdb $4 2>$tconfig && \
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t users --dbname=$tmpdb $4 2>$tconfig && \
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t mails --dbname=$tmpdb $4 2>$tconfig && \
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t mail_filters --dbname=$tmpdb $4 2>$tconfig && \
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t replication --dbname=$tmpdb $4 2>$tconfig && \
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t del_cdrs --dbname=$tmpdb $4 2>$tconfig && \
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t ldap_servers --dbname=$tmpdb $4 2>$tconfig && \
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t user_filters --dbname=$tmpdb $4 2>$tconfig && \
                            rm -f $tconfig
                            # ensure compatibility between V1 and V10
                            psql_result=`/usr/bin/sudo -u postgres /usr/bin/psql --dbname=$tmpdb -c "UPDATE filter_conditions SET name='cu.pbx' WHERE name='cdr_pbx' AND type='pbx';"`
                            if [ $? -ne 0 ]; then
                                log_error "Updating filter_conditions 1 failed: $psql_result"
                                db_error=1
                            fi
                            psql_result=`/usr/bin/sudo -u postgres /usr/bin/psql --dbname=$tmpdb -c "UPDATE filter_conditions SET name='cu.cn' WHERE name='cn' AND type='local';"`
                            if [ $? -ne 0 ]; then
                                log_error "Updating filter_conditions 2 failed: $psql_result"
                                db_error=1
                            fi
                            psql_result=`/usr/bin/sudo -u postgres /usr/bin/psql --dbname=$tmpdb -c "UPDATE mails SET group_by='cu.cn' WHERE group_by='cn';"`
                            if [ $? -ne 0 ]; then
                                log_error "Updating mails 1 failed: $psql_result"
                                db_error=1
                            fi
                            psql_result=`/usr/bin/sudo -u postgres /usr/bin/psql --dbname=$tmpdb -c "UPDATE mails SET sort_by='cu.cn' WHERE sort_by='cn';"`
                            if [ $? -ne 0 ]; then
                                log_error "Updating mails 2 failed: $psql_result"
                                db_error=1
                            fi
                            psql_result=`/usr/bin/sudo -u postgres /usr/bin/psql --dbname=$tmpdb -c "UPDATE mails SET pbx=(SELECT tmp.sys FROM (SELECT DISTINCT cu.sys, cu.id FROM cdr_users cu ORDER BY cu.id DESC) AS tmp LIMIT 1) WHERE pbx IS NULL OR pbx='';"`
                            if [ $? -ne 0 ]; then
                                log_error "Updating mails 3 failed: $psql_result"
                                db_error=1
                            fi
                        elif [ $5 == "innovaphone-exchange" ]; then
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t servers --dbname=$tmpdb $4 2>$tconfig && \
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t users --dbname=$tmpdb $4 2>$tconfig && \
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t calendar_items --dbname=$tmpdb $4 2>$tconfig && \
                            rm -f $tconfig
                        elif [ $5 == "innovaphone-faxserver" ]; then
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t servers --dbname=$tmpdb $4 2>$tconfig && \
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t server_locations --dbname=$tmpdb $4 2>$tconfig && \
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t server_strings --dbname=$tmpdb $4 2>$tconfig && \
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t fax_jobs --dbname=$tmpdb $4 2>$tconfig && \
                            rm -f $tconfig
                        elif [ $5 == "innovaphone" ]; then
                            /usr/bin/sudo -u postgres /usr/bin/pg_restore --schema=public -Fc -a -t relay_hosts --dbname=$tmpdb $4 2>$tconfig && \
                            rm -f $tconfig
                        fi    
                        # add just constraints
                        psql_result=`/usr/bin/sudo -u postgres psql -1 -t -q -f $add_constraints --dbname=$tmpdb -v ON_ERROR_STOP= 2>&1`
                        if [ $? -ne 0 ]; then
                            log_error "Readding constraints failed: $psql_result"
                            db_error=1
                        fi
                        psql_result=`/usr/bin/sudo -u postgres psql -1 -t -q -f /home/postgres/restore/get_add_index_statement.psql --dbname=$tmpdb -v ON_ERROR_STOP= > $add_indexes2`
                        if [ $? -ne 0 ]; then
                            log_error "Reading constraint indexes failed: $psql_result"
                            db_error=1
                        fi
                        implicit_indexes=`cat $add_indexes2`
                        while read line
                        do
                            if [[ "${implicit_indexes}" == *${line}* ]]; then
                                itmp=0
                            else
                                echo $line >> $new_indexes
                            fi
                        done < $add_indexes
                        psql_result=`/usr/bin/sudo -u postgres psql -1 -t -q -f $new_indexes --dbname=$tmpdb -v ON_ERROR_STOP= 2>&1`
                        if [ $? -ne 0 ]; then
                            log_error "Adding custom indexes failed: $psql_result"
                            db_error=1
                        fi
                        if [ -e $tconfig -o $db_error -eq 1 ]; then
                            log_error "restoring the database failed"
                            cat $tconfig
                            /usr/bin/sudo -u postgres dropdb $tmpdb
                            /usr/bin/sudo -u postgres /usr/bin/createdb --template=template0 --encoding=utf8 -U postgres $5 &>/dev/null
                            /usr/bin/sudo -u postgres psql -c "ALTER DATABASE \"$5\" OWNER TO \"$5\"" &>/dev/null
                            /usr/bin/sudo -u postgres psql -c "GRANT ALL PRIVILEGES ON DATABASE \"$5\" to \"$5\"" &>/dev/null
                            /usr/bin/sudo -u $5 /usr/bin/pg_restore --schema=public --clean --dbname=$5 $3 -U $5 2>$tconfig
                            if [ -s $tconfig ]; then
                                log_msg "Fatal Error. Check if any DB available"
                            else
                                log_msg "Database was unchanged"
                            fi
                        else
                            /usr/bin/sudo -u postgres psql -f $update_seq_file $tmpdb 2>/dev/null 1>$tbackup
                            num_updates=`cat $tbackup | grep setval | wc -l`
                            num_lines=`cat $update_seq_file | egrep -e "SELECT[ \t]+setval" | wc -l`
                            if [ $num_updates != $num_lines ]; then
                                log_error "Sequences could not be restored=$num_updates"
                                cat $tbackup
                                /usr/bin/sudo -u postgres dropdb $tmpdb
                                /usr/bin/sudo -u postgres /usr/bin/createdb --template=template0 --encoding=utf8 -U postgres $5 &>/dev/null
                                /usr/bin/sudo -u postgres psql -c "ALTER DATABASE \"$5\" OWNER TO \"$5\"" &>/dev/null
                                /usr/bin/sudo -u postgres psql -c "GRANT ALL PRIVILEGES ON DATABASE \"$5\" to \"$5\"" &>/dev/null
                                /usr/bin/sudo -u $5 /usr/bin/pg_restore --schema=public --clean --dbname=$5 $3 -U $5 2>$tconfig
                                if [ -s $tconfig ]; then
                                    log_msg "Fatal Error. Check if any DB available"
                                else
                                    log_msg "Database was unchanged"
                                fi
                            else
                                /usr/bin/sudo -u postgres psql -c "ALTER DATABASE \"$tmpdb\" RENAME TO \"$5\"" 1>/dev/null 2>$tconfig
                                if [ -s $tconfig ]; then
                                    echo -n "LOG: $(date '+%Y-%m-%d %H:%M:%S') - Fatal Error. Renaming database failed: "
                                    cat $tconfig
                                fi
                                /usr/bin/sudo -u postgres /usr/bin/psql --dbname=$5 -c "ANALYZE;" 1>/dev/null 2>$tconfig
                                if [ -s $tconfig ]; then
                                    echo -n "LOG: $(date '+%Y-%m-%d %H:%M:%S') - Warning: Analyze failed"
                                    cat $tconfig
                                fi
                            fi
                        fi
                        rm -f $add_constraints
                        rm -f $add_indexes
                        rm -f $add_indexes2
                        rm -f $new_indexes
                        rm -f $drop_states
                        rm -f $tbackup  
                        rm -f $tconfig  
                        /usr/bin/sudo -u postgres /etc/init.d/postgresql restart &>/dev/null
                        if [ $5 == "innovaphone-faxserver" ]; then
                            /etc/init.d/innovaphone-faxserver restart &>/dev/null
                        elif [ $5 == "innovaphone-reporting" -a "$reporting_lock" != "undefined" -a -f "$reporting_lock" ]; then  
                            rm -f $reporting_lock
                        fi
                    fi
                ;;
            esac
        fi
    ;;
    application)
        # $3 is the application name
        if [ "$2" != "" -a "$3" != "" ]; then
            check_and_lock_process "/tmp/config.sh.ap.postgres"
            if [ $? -ne 0 ]; then
                log_error "Couldn't lock $3 process. Exiting..."
                exit 19
            fi
            proc_arg0="/tmp/config.sh.ap.postgres"
            check_and_lock_process "/tmp/config.sh.application"
            if [ $? -ne 0 ]; then
                log_error "Couldn't lock $3 process. Exiting..."
                rm -f $proc_arg0
                exit 30
            fi
            proc_arg1="/tmp/config.sh.application" 
            case $2 in
                cfg-backup)
                    if [ "$4" != "" ]; then
                        shift && shift && shift
                        /bin/tar cf - $@ | gzip -fc6
                    fi
                ;;
                cfg-restore)
                   if [ $# -gt 4 ]; then
                      cd $5
                      for file in `find * -type f`; do
                          if [ -f "/${file}" ]; then
                              # Check the files exist in application.xml, otherwise drop them
                              if grep $file /var/www/innovaphone/apps/$4 > /dev/null
                              then
                                  /bin/cp `echo $file` $(echo "/${file}")
                              fi
                          fi 
                      done 
                      for directory in `find * -type d`; do
                          stat ${directory}/* > /dev/null 2>&1
                          if [ $? -eq 0 -a -d "/${directory}" ]; then
                              # Check the files exist in application.xml, otherwise drop them
                              if grep "<conf>/$directory</conf>" /var/www/innovaphone/apps/$4 &> /dev/null
                              then
                                  /bin/cp -r ${directory}/* $(echo "/${directory}")
                              fi
                          fi
                      done
                      shift && shift && shift && shift && shift
                      for actions in "$@"
                      do
                          $actions 2>&1 >/dev/null
                          shift
                      done
                   fi
                ;;
                script)
                    # Execute script provided by customer followed by arguments provided
                    if [ "$4" != "" ]; then
                        script=$4
                        shift && shift && shift && shift
                        $script "$@"
                    fi
                ;;
            esac
        fi
    ;;
    innovaphone)
    # argument 3: backup, restore
        if [ "$2" != "" ]; then
            check_and_lock_process "/tmp/config.sh.innovaphone"
            if [ $? -ne 0 ]; then
                log_error "Couldn't lock innovaphone process. Exiting..."
                exit 40
            fi
            proc_arg0="/tmp/config.sh.innovaphone"
            case $2 in
                cfg-backup)
                    if [ "$3" != "" ]; then
                        shift && shift
                        /bin/tar cf - $@ | gzip -fc6
                    fi
                ;;
                cfg-restore)
                   if [ "$3" != "" -a "$4" != "" ]; then
                      cert_server=`cat $3 | egrep -e "certServer[ =]+" | cut -d"=" -f2 | egrep -oe '\/[^"]*'`
                      cert_dir_trusted=`cat $3 | egrep -e "certDirTrusted[ =]+" | cut -d"=" -f2 | egrep -oe '\/[^"]*'`
                      cert_dir_rejected=`cat $3 | egrep -e "certDirRejected[ =]+" | cut -d"=" -f2 | egrep -oe '\/[^"]*'`
                      server_cfg_file=`cat $3 | egrep -e "serverCfgFile[ =]+" | cut -d"=" -f2 | egrep -oe '\/[^"]*'`
                      lighttpd_cfg=`cat $3 | egrep -e "lighttpdCfgFile[ =]+" | cut -d"=" -f2 | egrep -oe '\/[^"]*'`
                      cron_job=`cat $3 | egrep -e "cronInnovaphoneBackup[ =]+" | cut -d"=" -f2 | egrep -oe '\/[^"]*'`
                      ntp_conf=`cat $3 | egrep -e "ntpConf[ =]+" | cut -d"=" -f2 | egrep -oe '\/[^"]*'`
                      if [ "$cert_server" == "" -o "$cert_dir_trusted" == "" -o "$cert_dir_rejected" == "" -o "$server_cfg_file" == "" -o "$lighttpd_cfg" == "" -o "$cron_job" == "" -o "$ntp_conf" == "" ]; then
                          log_msg "Any of the config files could not be found"
                          log_msg "Cert Server: $cert_server"
                          log_msg "Dir Trusted: $cert_dir_trusted"
                          log_msg "Dir Rejected: $cert_dir_rejected"
                          log_msg "Cfg File: $server_cfg_file"
                          log_msg "Lighttpd Cfg: $lighttpd_cfg"
                          log_msg "Cron job: $cron_job"
                          log_msg "NTP config: $ntp_conf"
                          on_exit $proc_arg0 $proc_arg1
                          exit 41
                      fi
                      rm -f /tmp/config.stderr
                      if [ -d ${4}${cert_dir_trusted} ]; then
                          if [ ! -d $cert_dir_trusted ]; then
                              mkdir -p $cert_dir_trusted 2>/tmp/config.stderr
                          fi
                          cp ${4}${cert_dir_trusted}/* $cert_dir_trusted 2>/tmp/config.stderr
                          rm -f $cert_dir_rejected/* 2>/tmp/config.stderr
                      fi
                      if [ -f ${4}${cert_server} ]; then
                          cp ${4}${cert_server} $cert_server 2>/tmp/config.stderr
                          chmod 400 $cert_server 2>/tmp/config.stderr
                      fi
                      if [ -f ${4}${server_cfg_file} ]; then
                          cp ${4}${server_cfg_file} $server_cfg_file 2>/tmp/config.stderr
                          # Start rpcap if active in old config
                          if cat $server_cfg_file | grep "<rpcap>1</rpcap>" >/dev/null
                          then
                              num_proc=`ps auxwww | grep -v grep | grep rpcapd | wc -l`
                              if [ $num_proc -ne 2 ]; then
                                  if [ $num_proc -eq 0 ]; then
                                      /usr/local/bin/rpcapd -p 2002 -4 -n -d > /dev/null 2>&1
                                  else
                                      ps auxwww | grep -v grep | grep rpcapd | awk '{ print $2 }' | xargs kill -9
                                      /usr/local/bin/rpcapd -p 2002 -4 -n -d > /dev/null 2>&1
                                  fi
                              fi
                          else     
                              if ps auxwww | grep -v grep | grep rpcapd &>/dev/null
                              then
                                  ps auxwww | grep -v grep | grep rpcapd | awk '{ print $2 }' | xargs kill -9
                              fi
                          fi
                          # Old config backup
                          if cat $server_cfg_file | grep "<webdav>" >/dev/null
                          then
                              tmp_file=`mktemp /tmp/webdav.XXXXX`
                              if [ $? -eq 0 ]; then
                                  sed -e "s;<webdav>;<webdav write=\"true\">;g" < $server_cfg_file > $tmp_file
                                  cp $tmp_file $server_cfg_file
                                  rm $tmp_file
                              fi
                          fi
                          if [ ! -f ${4}${cron_job} ]; then
                              if cat $server_cfg_file | egrep -e "<url poll=\"[0-9]+\">[^<]+</url>" >/dev/null
                              then
                                  poll_time=`cat $server_cfg_file | egrep -oe "<url poll=\"[0-9]+" | tr -d " " | cut -d'"' -f2`
                                  echo "# /etc/cron.d/innovaphone_backup" > /etc/cron.d/innovaphone_backup
                                  echo "*/${poll_time} * * * * root [ -x /usr/bin/sudo ] && /usr/bin/sudo -u www-data php5-cgi /var/www/innovaphone/backup.php >> /var/www/innovaphone/log/backup.log 2>&1" >> /etc/cron.d/innovaphone_backup
                                  /etc/init.d/cron restart 1>/dev/null 2>/tmp/config.stderr
                              fi
                          fi 
                      fi
                      if [ -f ${4}${cron_job} ]; then
                          cp ${4}${cron_job} $cron_job 2>/tmp/config.stderr
                          /etc/init.d/cron restart 1>/dev/null 2>/tmp/config.stderr
                      fi
                      if [ -f ${4}${ntp_conf} ]; then
                          cp ${4}${ntp_conf} $ntp_conf 2>/tmp/config.stderr
                          /etc/init.d/ntp restart 1>/dev/null 2>/tmp/config.stderr
                      fi
                      if [ -f ${4}${lighttpd_cfg} ]; then
                          cp ${4}${lighttpd_cfg} $lighttpd_cfg 2>/tmp/config.stderr
                      fi
                      cp $lighttpd_cfg /home/lighttpd/server_config/lighttpd.conf 2>/tmp/config.stderr
                      chown www-data:www-data /home/lighttpd/server_config/lighttpd.conf 2>/tmp/config.stderr
                      if [ -s /tmp/config.stderr ]; then
                          log_error "Error restoring the config"
                          cat /tmp/config.stderr
                      fi
                   fi
                ;;
            esac
        fi
    ;;
    log-files)
        if [ "$2" == "" -o "$3" == "" ]; then
            log_error "no arguments provided"
            on_exit $proc_arg0 $proc_arg1
            exit 70
        fi
        if [ "$2" == "post" ]; then
            url=$3
            cert_dir=$4
            shift && shift && shift && shift
            if [ $# -gt 4 ]; then
                dummy=`echo $url | egrep -oe "[^/]*$"`
                touch /tmp/$dummy
                if [ ! -f "/tmp/$dummy" ]; then
                    log_error "Can't create temp file. Exiting..."
                    on_exit $proc_arg0 $proc_arg1
                    exit 71
                fi
                ret=`/usr/bin/curl -f -i -s -S --url $url -T /tmp/$dummy -H "Expect: " --anyauth --location-trusted --max-redirs 4 --capath $cert_dir 2>&1 | \
                     sed -e "s;curl:[ \t]*(\([0-9]*\).*$;\1;"`
                rm -f /tmp/$dummy
                if [ "$ret" == "60" ]; then
                    echo $ret
                    on_exit $proc_arg0 $proc_arg1
                    exit 72
                elif [ "$ret" == "51" ]; then
                    ret=`/bin/tar -c --ignore-failed-read -f - $@ | \
                         gzip -fc6 | \
                         /usr/bin/curl -f -i -s -S --url $url -T - -H "Expect: " --anyauth --location-trusted --max-redirs 4 -k 2>&1 | \
                         egrep -e "curl:[ \t]*([0-9]*)" | \
                         sed -e "s;curl:[ \t]*(\([0-9]*\).*$;\1;"`
                    echo -n $ret
                else
                    ret=`/bin/tar -c --ignore-failed-read -f - $@ | \
                         gzip -fc6 | \
                         /usr/bin/curl -f -i -s -S --url $url -T - -H "Expect: " --anyauth --location-trusted --max-redirs 4 --capath $cert_dir 2>&1 | \
                         egrep -e "curl:[ \t]*([0-9]*)" | \
                         sed -e "s;curl:[ \t]*(\([0-9]*\).*$;\1;"`
                    echo -n $ret
                fi
            fi
        elif [ "$2" == "echo" ]; then
            shift && shift
            /bin/tar --ignore-failed-read -cf - $@ | gzip -fc6
        else
            log_error "unknown HTTP method"
            on_exit $proc_arg0 $proc_arg1
            exit 73
        fi
    ;;
    # check)
    #    ps auxwww | egrep -e "(postgres (db-restore|db-backup)|application (cfg-restore|script)).*$2" | grep -v "egrep" | wc -l
    #;;
    #check-n)
    #    ps auxwww | egrep -e "(postgres (db-restore|db-backup)|application (cfg-restore|script)).*$2" | grep -v "egrep" | awk '{print $13" "$14}'
    #;;
    packages)
        if [ "$2" != "" ]; then
            deb_file=$2
            shift && shift
            echo "Installed Debian Packages" > $deb_file
            while [ $# -gt 0 ]
            do
                echo "$1 $2" >> $deb_file
                shift && shift
            done
        fi
    ;;
esac

on_exit $proc_arg0 $proc_arg1 $reporting_lock
